#!/bin/env tclsh


source /data/mmti/src/mmtitcl/starbase.tcl

set allcols "";  # This is the list of all column names

set debug 0

proc debug { arg } {
    if $::debug {
	puts stderr $arg
    }
}

proc sextodecimal { pos } {
    set l [ split $pos : ]
    set d [string trimleft [lindex $l 0] "0"]
    set m [string trimleft [lindex $l 1] "0"]
    set s [string trimleft [lindex $l 2] "0"]
    if { $m == "" } { set m 0 }
    if { $s == "" } { set s 0 }
    if  { [string index [string trim $d] 0] == "-" } {
        set sign -1
        set d [expr -($d)]
    } else {
        set sign 1
    }
    return [expr $sign * ($d + $m/60. + $s/3600.) ]
}

proc stripargs {arglist argkey} {
    foreach key $argkey {
	set varname [lindex $key 1]
	upvar $varname var
	set var [lindex $key 2]             ;# Default value
        set flag [lindex $key 0]
        set index [lsearch -exact $arglist $flag]
        set index1 [expr $index + 1]
        if {$index >= 0} {
            set var [lindex $arglist $index1]
            set arglist [lreplace $arglist $index $index1]
        }
	catch { set var [expr $var] }
    }
    return $arglist
}

#
# Returns the group id if we have a match, otherwise returns
# the index in the list where a new group id should be inserted.
# The index is returned as a negative number -(index)-1
#
proc search_grouplist { key } {
    global g s

debug "\nSearching for $key with index $g(index)"

    set newindex 0

    if { $g(length) == 0 } { return -1 } 

    set v0 [lindex $key 0]
    set g0 $g([lindex $g(list) $g(index)],0)

debug "Starting v0=$v0 g0=$g0"    

    while { $g(index) > 0 && $g0 > $v0 - $s(0)/2 } {
	incr g(index) -1
	set g0 $g([lindex $g(list) $g(index)],0)
	debug $g(index)
    }

debug "Here1 -- g(index)=$g(index) newindex=$newindex"

    while { $g0 < $v0 - $s(0)/2 } {
	incr g(index) 1
	if { $v0 > $g0 } { set newindex $g(index) } 
	if { $g(index) > $g(length)-1 } { return [expr -$g(index)-1] }
	set g0 $g([lindex $g(list) $g(index)],0)
	debug $g(index)
    }

debug "Here2-- g(index)=$g(index) newindex=$newindex"
    
    while { $g0 < $v0 + $s(0)/2 } {
	debug "Doing full comparison for $key"
	set bad 0
	for { set i 1 } { $i < $::n } { incr i }  {

	    set vi [lindex $key $i]
	    set gi $g([lindex $g(list) $g(index)],$i)
	    debug "Comparing i=$i, $vi to $gi with tolerance [expr $s($i)/2]"	    
	    if { abs($vi - $gi) > $s($i)/2 } { set bad 1 ; break }
	}
	debug "Bad=$bad"
	if { $bad == 0 } { return [lindex $g(list) $g(index)] }
	incr g(index) 1
	if { $v0 > $g0 } { set newindex $g(index) } 
	if { $g(index) > $g(length)-1 } { return [expr -$g(index)-1] }
	set g0 $g([lindex $g(list) $g(index)],0)
	debug $g(index)
    }

debug "Here3-- g(index)=$g(index) newindex=$newindex"

    while { $g(index) < $g(length)-1 } {
	if { $g0 < $v0 } { set newindex $g(index) }
	incr g(index)
	set g0 $g([lindex $g(list) $g(index)],0)
	debug $g(index)
    }
debug "Here4-- g(index)=$g(index) newindex=$newindex"

debug "Newindex $newindex for $v0 in $g(list)"
    return [expr -$newindex-1]

}

proc addtable { T file } {
    upvar $T G
    starbase_read G $file
    global g

    for { set i 0 } { $i < $::n } { incr i } {
	lappend collist [starbase_colnum G [set ::m($i)]]
    }

    debug "collist: $collist"

    # Add any new columns to the master list of columns to be output
    foreach c [starbase_columns G] {
	if { [lsearch -exact $::allcols $c] == -1 } {
	    lappend ::allcols $c
	}
    }

    starbase_foreachrow G {
	set key ""
	foreach col $collist {
	    lappend key [sextodecimal $G($row,$col)]
	}
	set gid [search_grouplist $key]
	if { $gid >= 0 } {
	    set l [list $file $row]
	    lappend g($gid,members) $l
	    set g($file,$row) $gid
	    debug "Adding $file, row $row to group $gid"
	} else {
	    set place [expr -($gid+1)]
	    set gid $g(length)
	    set g(list) [linsert $g(list) $place $gid]
	    set l [list $file $row]
	    lappend g($gid,members) $l
	    set g($file,$row) $gid
	    for { set i 0 } { $i < $::n } { incr i } {
		set g($gid,$i) [lindex $key $i]
	    }
	    incr g(length)
	    debug "Creating new group $gid for $file row $row"
	    debug "g(list) $g(list)"
	}
    }
}
		
proc printtable { } {
    set newid 0
    set cols $::allcols
    global tablelist Obs g

    # Make the table header
    lappend cols Group Obs
    set header [join $cols \t]
    regsub -all {[^\t]} $header - dashline
    puts $header
    puts $dashline

    foreach table $tablelist {
	upvar $table T
	starbase_foreachrow T {
	    set gid $g($table,$row)
	    debug "#Members of $gid: [llength $g($gid,members)]"

	    # Only output groups with 2 or more members
	    if { [llength $g($gid,members)] < 2 } continue

	    # Assign new sequential group ids
	    if {![info exists g($gid,newid)]} {
		set g($gid,newid) [incr newid]
	    }

	    foreach c $::allcols {
		set colnum [starbase_colnum T $c]
		if { $colnum == 0 } {
		    puts -nonewline "\t"
		} else {
		    puts -nonewline "$T($row,$colnum)\t"
		}
	    }
	    puts "$g($gid,newid)\t$Obs($table)"
	}
    }
}


set argv [stripargs $argv {
    {-n  n  2 }
    {-m0 m(0) ra }
    {-s0 s(0) 1/3600./15. }
    {-m1 m(1) dec }
    {-s1 s(1) 1/3600. }
    {-m2 m(2) dec }
    {-s2 s(2) 2.7e-4 }
    {-m3 m(3) dec }
    {-s3 s(3) 2.7e-4 }
    {-m4 m(4) dec }
    {-s4 s(4) 2.7e-4 }
    {-m5 m(5) dec }
    {-s5 s(5) 2.7e-4 }
}]

debug $argv
debug $n

set g(length) 0
set g(index) 0
set g(list) {}
set Obsx 0

foreach file $argv {
    incr Obsx
    set Obs($file) $Obsx
    addtable $file $file
    lappend tablelist $file
    puts stderr "Completed $file"
}

printtable

foreach name [array names g] {
    debug "$name $g($name)"
}
